@@ -7,6 +7,7 @@ use PRANG::Graph;
 use PRANG::Cookbook::Role::Date;
 use PRANG::Cookbook::Role::Time;
 
-with 'PRANG::Cookbook::Role::Date', 'PRANG::Cookbook::Role::Time', 'PRANG::Cookbook::Node';
+with 'PRANG::Cookbook::Role::Date', 'PRANG::Cookbook::Role::Time',
+	'PRANG::Cookbook::Node';
 
 1;
