@@ -2,7 +2,6 @@
 package PRANG::Cookbook::Library;
 
 use Moose;
-use MooseX::Method::Signatures;
 use PRANG::Graph;
 use PRANG::XMLSchema::Types;
 
@@ -11,9 +10,10 @@ has_element 'book' =>
 	is => 'rw',
 	isa => 'ArrayRef[PRANG::Cookbook::Book]',
 	xml_required => 1,
+	required => 1,
 	;
 
-sub root_element { 'library' }
+sub root_element {'library'}
 with 'PRANG::Cookbook';
 
 1;
