@@ -2,7 +2,6 @@
 package PRANG::Cookbook::Note;
 
 use Moose;
-use MooseX::Method::Signatures;
 use PRANG::Graph;
 use PRANG::XMLSchema::Types;
 
@@ -20,6 +19,7 @@ has_element 'from' =>
 	is => 'rw',
 	isa => 'Str',
 	xml_required => 1,
+	required => 1,
 	;
 
 has_element 'to' =>
@@ -27,6 +27,7 @@ has_element 'to' =>
 	is => 'rw',
 	isa => 'Str',
 	xml_required => 1,
+	required => 1,
 	;
 
 has_element 'sent' =>
@@ -48,6 +49,7 @@ has_element 'subject' =>
 	is => 'rw',
 	isa => 'Str',
 	xml_required => 1,
+	required => 1,
 	;
 
 has_element 'body' =>
@@ -58,7 +60,7 @@ has_element 'body' =>
 	xml_required => 0,
 	;
 
-sub root_element { 'note' }
+sub root_element {'note'}
 with 'PRANG::Cookbook';
 
 1;
