@@ -2,11 +2,12 @@
 package PRANG::Marshaller;
 
 use Moose;
-use MooseX::Method::Signatures;
+use MooseX::Params::Validate;
 use Moose::Util::TypeConstraints;
 
 use XML::LibXML 1.65;
 use PRANG::Util qw(types_of);
+use Carp;
 
 BEGIN {
 	class_type 'Moose::Meta::Class';
@@ -14,7 +15,7 @@ BEGIN {
 	class_type "XML::LibXML::Element";
 	class_type "XML::LibXML::Node";
 	role_type "PRANG::Graph";
-};
+}
 
 has 'class' =>
 	isa => "Moose::Meta::Class|Moose::Meta::Role",
@@ -22,17 +23,29 @@ has 'class' =>
 	required => 1,
 	handles => [qw(marshall_in_element to_libxml)],
 	trigger => sub {
-		my $self = shift;
-		my $class = $self->class;
-		if ( !$class->can("marshall_in_element") ) {
-			$class = $class->name if ref $class;
-			die "Can't marshall $class; didn't 'use PRANG::Graph' ?";
-		}
+	my $self = shift;
+	my $class = $self->class;
+	if ( !$class->can("marshall_in_element") ) {
+		$class = $class->name if ref $class;
+		die "Can't marshall $class; didn't 'use PRANG::Graph' ?";
+	}
 	},
 	;
+	
+has 'encoding' =>
+    isa => 'Str',
+    is => 'ro',
+    default => 'UTF-8';
 
 our %marshallers;  # could use MooseX::NaturalKey?
-method get($inv: Str $class) {
+
+sub get {
+    my $inv = shift;
+    my ( $class ) = pos_validated_list(
+        \@_,
+        { isa => 'Str' },
+    );      
+    
 	if ( ref $inv ) {
 		$inv = ref $inv;
 	}
@@ -47,12 +60,16 @@ method get($inv: Str $class) {
 			die "cannot marshall $class; no ->meta";
 	};
 	my $meta = $class->meta;
-	if ( $meta->does_role("PRANG::Graph") or
-		     $meta->meta->does_role("PRANG::Graph::Meta::Class")
-		    ) {
+	if ($meta->does_role("PRANG::Graph")
+		or
+		$meta->meta->does_role("PRANG::Graph::Meta::Class")
+		)
+	{
+	    
+	    my $encoding = $class->can('encoding') ? $class->encoding : 'UTF-8';
 		$marshallers{$class} ||= do {
-			$inv->new( class => $class->meta );
-		}
+			$inv->new( class => $class->meta, encoding => $encoding );
+			}			
 	}
 	else {
 		die "cannot marshall ".$meta->name
@@ -60,9 +77,23 @@ method get($inv: Str $class) {
 	}
 }
 
-method parse( Str $xml ) {
+sub parse {
+    my $self = shift;
+    my ( $xml, $filename, $fh, $lax ) = validated_list(
+        \@_,
+        xml => { isa => 'Str', optional => 1 },
+        filename => { isa => 'Str', optional => 1 },
+        fh => { isa => 'GlobRef', optional => 1 },
+        lax => { isa => 'Bool', optional => 1, default => 0 },
+    );    
 
-	my $dom = XML::LibXML->new->parse_string($xml);
+	my $parser = XML::LibXML->new;
+	my $dom = (
+		defined $xml ? $parser->parse_string($xml) :
+			defined $filename ? $parser->parse_file($filename) :
+			defined $fh ? $parser->parse_fh($fh) :
+			croak("no input passed to parse")
+	);
 
 	my $rootNode = $dom->documentElement;
 	my $rootNodeNS = $rootNode->namespaceURI;
@@ -73,9 +104,12 @@ method parse( Str $xml ) {
 		my $found;
 		my $root_localname = $rootNode->localname;
 		my @expected;
-		for my $class ( @possible ) {
-			if ( $root_localname eq
-				     $class->name->root_element ) {
+		for my $class (@possible) {
+			if ($root_localname eq
+				$class->name->root_element
+				)
+			{
+
 				# yeah, this is lazy ;-)
 				$self = (ref $self)->get($class->name);
 				$found = 1;
@@ -93,11 +127,16 @@ method parse( Str $xml ) {
 	my $expected_ns = $self->class->name->xmlns;
 	if ( $rootNodeNS and $expected_ns ) {
 		if ( $rootNodeNS ne $expected_ns ) {
-			die "Namespace mismatch: expected '$expected_ns', found '$rootNodeNS'";
+			die
+"Namespace mismatch: expected '$expected_ns', found '$rootNodeNS'";
 		}
 	}
-	if ( !defined($rootNode->prefix) and
-		     !defined($rootNode->getAttribute("xmlns")) ) {
+	if (!defined($rootNode->prefix)
+		and
+		!defined($rootNode->getAttribute("xmlns"))
+		)
+	{
+
 		# namespace free;
 		$xsi->{""}="";
 	}
@@ -107,30 +146,38 @@ method parse( Str $xml ) {
 		xpath => "",
 		xsi => $xsi,
 		prefix => "",
-	       );
+	);
 
 	my $rv = $self->class->marshall_in_element(
 		$rootNode,
 		$context,
-	       );
+		$lax,
+	);
 	$rv;
 }
 
-method xml_version { "1.0" };
-method encoding { "UTF-8" };
+sub xml_version { "1.0" }
 
 # nothing to see here ... move along please ...
 our $zok;
 our %zok_seen;
-our @zok_themes = (qw( tmnt octothorpe quantum pokemon hhgg pasta
-		       phonetic sins punctuation discworld lotr
-		       loremipsum batman tld garbage python pooh
-		       norse_mythology ));
+our @zok_themes = (
+	qw( tmnt octothorpe quantum pokemon hhgg pasta
+		phonetic sins punctuation discworld lotr
+		loremipsum batman tld garbage python pooh
+		norse_mythology )
+);
 our $zok_theme;
 
 our $gen_prefix;
 
-method generate_prefix( Str $xmlns ) {
+sub generate_prefix {
+    my $self = shift;
+    my ( $xmlns ) = pos_validated_list(
+        \@_,
+        { isa => 'Str' },
+    );    
+    
 	if ( $zok or eval { require Acme::MetaSyntactic; 1 } ) {
 		my $name;
 		do {
@@ -143,12 +190,12 @@ method generate_prefix( Str $xmlns ) {
 					}
 				}
 				else {
-					$zok_theme = int(time / 86400)
+					$zok_theme = int(time() / 86400)
 						% scalar(@zok_themes);
 				}
 				Acme::MetaSyntactic->new(
 					$zok_themes[$zok_theme],
-				       );
+				);
 			};
 			do {
 				$name = $zok->name;
@@ -156,57 +203,75 @@ method generate_prefix( Str $xmlns ) {
 					undef($zok);
 					undef($name);
 					goto next_theme;
-				};
-			} while ( length($name) > 10 or
-					  $name !~ m{^[A-Za-z]\w+$} );
-			next_theme:
-		}
+				}
+				} while (
+				length($name) > 10
+				or
+				$name !~ m{^[A-Za-z]\w+$}
+				);
+		next_theme:
+			}
 			until ($name);
 		return $name;
 	}
 	else {
+
 		# revert to a more boring prefix :)
 		$gen_prefix ||= "a";
 		$gen_prefix++;
 	}
 }
 
-method to_xml_doc( PRANG::Graph $item ) {
+sub to_xml_doc {
+    my $self = shift;
+    my ( $item ) = pos_validated_list(
+        \@_,
+        { isa => 'PRANG::Graph' },
+    );    
+    
 	my $xmlns = $item->xmlns;
 	my $prefix = "";
 	if ( $item->can("preferred_prefix") ) {
 		$prefix = $item->preferred_prefix;
 	}
 	my $xsi = { $prefix => ($xmlns||"") };
+
 	# whoops, this is non-reentrant
 	%zok_seen=();
 	undef($gen_prefix);
 	my $doc = XML::LibXML::Document->new(
 		$self->xml_version, $self->encoding,
-	       );
+	);
 	my $root = $doc->createElement(
 		($prefix ? "$prefix:" : "" ) .$item->root_element,
-	       );
-	if ( $xmlns ) {
+	);
+	if ($xmlns) {
 		$root->setAttribute(
 			"xmlns".($prefix?":$prefix":""),
 			$xmlns,
-		       );
+		);
 	}
-	$doc->setDocumentElement( $root );
+	$doc->setDocumentElement($root);
 	my $ctx = PRANG::Graph::Context->new(
 		xpath => "/".$root->nodeName,
 		base => $self,
 		prefix => $prefix,
 		xsi => $xsi,
-	       );
+	);
 	$item->meta->to_libxml( $item, $root, $ctx );
 	$doc;
 }
 
-method to_xml( PRANG::Graph $item ) {
+sub to_xml {
+    my $self = shift;
+    my ( $item, $format ) = pos_validated_list(
+        \@_,
+        { isa => 'PRANG::Graph' },
+        { isa => 'Int', default => 0 },
+    );    
+    
 	my $document = $self->to_xml_doc($item);
-	$document->toString;
+	$document->toString($format);
 }
 
 1;
